#include <stdio.h>
#include <string.h>

#include "FreeRTOS.h"
#include "freerotsTask.h"
#include "task.h"

#include "pDebug.h"
#include "upUartParse.h"
#include "FM_GPIO.h"
#include "FM_ADC.h"
#include "queueUart.h"
#include "parameter.h"



/* ȡĴʼλ */
#define maxStartReadResAddr 10
/* ȡĴ󳤶 */
#define maxReadResAddrLen 20

/* дĴʼλ */
#define maxStartWriteResAddr 10
/* дĴ󳤶 */
#define maxWriteResAddrLen 20

/* ζȡļ */
#define maxReadCfgLen 80

/* дļֽ */
#define maxDistributionCfgLen 230

/* 2Ľ */
#define tick_2S (configTICK_RATE_HZ * 2)
/* 1ӵĽ */
#define tick_1M (configTICK_RATE_HZ * 60)

/* ļͱ */
typedef enum {
    /* (1ֽ)ģͨŲ(00Hʹøô,01H4800;02H9600;03H19200;04H38400;05H57600;06H115200)  */
    Upward_UART_Communication_Baud_Rate     = 0x0001,
    J1_485_Communication_Baud_Rate          = 0x0002,
    J2_485_Communication_Baud_Rate          = 0x0003,
    J3_485_Communication_Baud_Rate          = 0x0004,
    J4_485_Communication_Baud_Rate          = 0x0005,
    J5_0_485_Communication_Baud_Rate        = 0x0006,
}gateWayCfgFileType;

/* ״̬ */
typedef enum {
    wait                        = 0,    /* ״̬ʼ״̬ */
    startFlagGW,                        /* յ֡ͷ */
    functionCodeGW,                     /* յ */
    readRegStartAddressGW,              /* յȡĴʼַ */
    readRegStartNumberGW,               /* յȡĴ */
    crcCheckBitGW,                      /* յУλ */
    endFlagGW,                          /* յ֡β */
    writeRegStartAddressGW,             /* յдĴʼַ */
    writeRegStartNumberGW,              /* յдĴ */
    cfgFramesNumGW,                     /* յļ֡ */
    cfgLengthGW,                        /* յļ */
    readCfgLengthGW,                    /* յȡļ */
    startFlagSL,                        /* յ֡ͷ */
    portSL,                             /* յ˿ں */
    lengthSL,                           /* յݰĳ */
    endFlagSL,                          /* յ֡β */
} uartStateMachine;


// /* ʱ1minûļǰ */
// static uint32_t gw485CfgTime = 0;

/* ʱ2ûһ֡ݣǰ */
static uint32_t gw485RxTime = 0;

/* gw485 */
static uint8_t gw485RxBuffer[1100];
static uint16_t gw485RxBufferIndex = 0;

/* ״̬״̬ */
static uartStateMachine state = wait;
/* ֡ */
static uint16_t frameLength = 0;

static void stateMachine(device_handle device);

/* ״̬ */
static uint8_t analysisWait(void);
static uint8_t analysisStartFlagGW(void);
static uint8_t analysisFunctionCodeGW(void);
static uint8_t analysisReadRegStartAddressGW(void);
static uint8_t analysisReadRegStartNumberGW(void);
static uint8_t analysisCrcCheckBitGW(void);
static uint8_t analysisEndFlagGW(device_handle device);
static uint8_t analysisWriteRegStartAddressGW(void);
static uint8_t analysisWriteRegStartNumberGW(void);
static uint8_t analysisCfgFramesNumGW(void);
static uint8_t analysisCfgLengthGW(void);
static uint8_t analysisReadCfgLengthGW(void);
static uint8_t analysisStartFlagSL(void);
static uint8_t analysisPortSL(void);
static uint8_t analysisLengthSL(void);
static uint8_t analysisEndFlagSL(void);




static void GW_MsgProcFunc_Read_Register(device_handle device,uint8_t *buff, uint16_t buffLen);
static void GW_MsgProcFunc_Write_Register(device_handle device, void *pMsg, uint32_t MsgLen);
static void GW_MsgProcFunc_Distribution_Profile(device_handle device, void *pMsg, uint32_t MsgLen);
static void GW_MsgProcFunc_Read_Profile(device_handle device, void *pMsg, uint32_t MsgLen);


/* Ĵ */
typedef uint16_t (*RegProcFunc)(void*);
typedef struct _SL_RegProcTable{
    uint32_t     regId;
    RegProcFunc  pRegProc;
}SL_RegProcTable;

/* Ĵ */
static uint16_t ReadRegisterWorkCurrent(void *pMsg);
static uint16_t ReadRegisterWorkVoltage(void *pMsg);
static uint16_t ReadRegisterTemperature(void *pMsg);
static uint16_t ReadRegisterPort1_Power(void *pMsg);
static uint16_t ReadRegisterPort2_Power(void *pMsg);
static uint16_t ReadRegisterPort3_Power(void *pMsg);
static uint16_t ReadRegisterPort4_Power(void *pMsg);
static uint16_t ReadRegisterPort5_Power(void *pMsg);
static uint16_t ReadRegisterPort6_Power(void *pMsg);
static uint16_t ReadRegisterPort7_Power(void *pMsg);
static uint16_t ReadRegisterPort8_Power(void *pMsg);
static uint16_t ReadRegisterPort9_Power(void *pMsg);
static uint16_t WriteRegisterPort1_Power(void *pMsg);
static uint16_t WriteRegisterPort2_Power(void *pMsg);
static uint16_t WriteRegisterPort3_Power(void *pMsg);
static uint16_t WriteRegisterPort4_Power(void *pMsg);
static uint16_t WriteRegisterPort5_Power(void *pMsg);
static uint16_t WriteRegisterPort6_Power(void *pMsg);
static uint16_t WriteRegisterPort7_Power(void *pMsg);
static uint16_t WriteRegisterPort8_Power(void *pMsg);
static uint16_t WriteRegisterPort9_Power(void *pMsg);

/* ȡĴ */
SL_RegProcTable g_RegTblR[] =
{    
    {GW_Register_WorkCurrent,    ReadRegisterWorkCurrent},
    {GW_Register_WorkVoltage,    ReadRegisterWorkVoltage},
    {GW_Register_Temperature,    ReadRegisterTemperature},
    {GW_Register_Port1_Power,    ReadRegisterPort1_Power},
    {GW_Register_Port2_Power,    ReadRegisterPort2_Power},
    {GW_Register_Port3_Power,    ReadRegisterPort3_Power},
    {GW_Register_Port4_Power,    ReadRegisterPort4_Power},
    {GW_Register_Port5_Power,    ReadRegisterPort5_Power},
    {GW_Register_Port6_Power,    ReadRegisterPort6_Power},
    {GW_Register_Port7_Power,    ReadRegisterPort7_Power},
    {GW_Register_Port8_Power,    ReadRegisterPort8_Power},
    {GW_Register_Port9_Power,    ReadRegisterPort9_Power},
};

/* дĴ */
SL_RegProcTable g_RegTblW[] =
{
    {GW_Register_Port1_Power,    WriteRegisterPort1_Power},
    {GW_Register_Port2_Power,    WriteRegisterPort2_Power},
    {GW_Register_Port3_Power,    WriteRegisterPort3_Power},
    {GW_Register_Port4_Power,    WriteRegisterPort4_Power},
    {GW_Register_Port5_Power,    WriteRegisterPort5_Power},
    {GW_Register_Port6_Power,    WriteRegisterPort6_Power},
    {GW_Register_Port7_Power,    WriteRegisterPort7_Power},
    {GW_Register_Port8_Power,    WriteRegisterPort8_Power},
    {GW_Register_Port9_Power,    WriteRegisterPort9_Power},
};

/**
  * @brief  ȡĴ
  * @param
  * @retval
  */
uint16_t ReadRegisterWorkCurrent(void *pMsg)
{
    return (uint16_t)(getWorkCurrent() * 10);
}

/**
  * @brief  ȡѹĴ
  * @param
  * @retval
  */
uint16_t ReadRegisterWorkVoltage(void *pMsg)
{
    return (uint16_t)(getWorkVoltage() * 10);
}

/**
  * @brief  ȡ¶ȼĴ
  * @param
  * @retval
  */
uint16_t ReadRegisterTemperature(void *pMsg)
{
    return (uint16_t)(getTemperature() * 10);
}

/**
  * @brief  ȡ˿1Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort1_Power(void *pMsg)
{
    return readPwrCtrlState(J1_PwrCtrl);
}

/**
  * @brief  ȡ˿2Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort2_Power(void *pMsg)
{
    return readPwrCtrlState(J2_PwrCtrl);
}

/**
  * @brief  ȡ˿3Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort3_Power(void *pMsg)
{
    return readPwrCtrlState(J3_PwrCtrl);
}

/**
  * @brief  ȡ˿4Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort4_Power(void *pMsg)
{
    return readPwrCtrlState(J4_PwrCtrl);
}

/**
  * @brief  ȡ˿5Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort5_Power(void *pMsg)
{
    return readPwrCtrlState(J5_PwrCtrl);
}

/**
  * @brief  ȡ˿6Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort6_Power(void *pMsg)
{
    return readPwrCtrlState(J6_PwrCtrl);
}

/**
  * @brief  ȡ˿7Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort7_Power(void *pMsg)
{
    return readPwrCtrlState(J7_PwrCtrl);
}

/**
  * @brief  ȡ˿8Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort8_Power(void *pMsg)
{
    return readPwrCtrlState(J8_PwrCtrl);
}

/**
  * @brief  ȡ˿9Դ״̬
  * @param
  * @retval
  */
uint16_t ReadRegisterPort9_Power(void *pMsg)
{
    return readPwrCtrlState(J9_PwrCtrl);
}

/**
  * @brief  ö˿1Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort1_Power(void *pMsg)
{
    writePwrCtrlState(J1_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    log_info("state %d\n", (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿2Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort2_Power(void *pMsg)
{
    writePwrCtrlState(J2_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿3Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort3_Power(void *pMsg)
{
    writePwrCtrlState(J3_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿4Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort4_Power(void *pMsg)
{
    writePwrCtrlState(J4_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿5Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort5_Power(void *pMsg)
{
    writePwrCtrlState(J5_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿6Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort6_Power(void *pMsg)
{
    writePwrCtrlState(J6_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿7Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort7_Power(void *pMsg)
{
    writePwrCtrlState(J7_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿8Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort8_Power(void *pMsg)
{
    writePwrCtrlState(J8_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}

/**
  * @brief  ö˿9Դ״̬
  * @param
  * @retval
  */
uint16_t WriteRegisterPort9_Power(void *pMsg)
{
    writePwrCtrlState(J9_PwrCtrl, (uint8_t)(*(uint16_t *)pMsg));
    return 0;
}


/**
  * @brief  ״̬ wait
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define waitMaxLen 2
uint8_t analysisWait(void)
{
    if (gw485RxBufferIndex >= 2) {
        /* ͸֡ͷ */
        if (gw485RxBuffer[0] == 'S' && gw485RxBuffer[1] == 'L') {
            // log_info("startFlagSL\n");
            state = startFlagSL;
            return TRUE;
        }

        /* ֡ͷ */
        else if (gw485RxBuffer[0] == 'G' && gw485RxBuffer[1] == 'W') {
            state = startFlagGW;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < waitMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ startFlagGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisStartFlagGW(void)
{
    if (gw485RxBuffer[2] == SL_Function_Code_Read_Register
        || gw485RxBuffer[2] == SL_Function_Code_Write_Register
        || gw485RxBuffer[2] == SL_Function_Code_Distribution_Profile
        || gw485RxBuffer[2] == SL_Function_Code_Read_Profile) {
            state = functionCodeGW;
            return TRUE;
    }

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ functionCodeGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define functionCodeGWMaxLen 5
uint8_t analysisFunctionCodeGW(void)
{
    if (gw485RxBufferIndex >= 5 && gw485RxBuffer[2] == SL_Function_Code_Read_Register) {
        /* ȡĴʼַ */
        uint16_t startResAddr = ((gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);
        if (startResAddr < maxStartReadResAddr) {
            frameLength = 10;
            state = readRegStartAddressGW;
            return TRUE;
        }
    }
    
    if (gw485RxBufferIndex >= 5 && gw485RxBuffer[2] == SL_Function_Code_Write_Register) {
        /* дĴʼַ */
        uint16_t startResAddr = ((gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);
        if (startResAddr < maxStartWriteResAddr) {
            state = writeRegStartAddressGW;
            return TRUE;
        }
    }

    // if (gw485RxBufferIndex >= 5 && gw485RxBuffer[2] == SL_Function_Code_Write_Register) {
    //     /* дĴʼַ */
    //     uint16_t startResAddr = ((gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);
    //     if (startResAddr < maxStartWriteResAddr) {
    //         state = writeRegStartAddressGW;
    //         return TRUE;
    //     }
    // }

    /* ļ·֡ */
    if ((gw485RxBufferIndex >= 5) && (gw485RxBuffer[2] == SL_Function_Code_Distribution_Profile)) {
        if (gw485RxBuffer[3] <= gw485RxBuffer[4]) {
            state = cfgFramesNumGW;
            return TRUE;
        }        
    }

    /* ļȡݵĳ */
    if ((gw485RxBufferIndex >= 5) && (gw485RxBuffer[2] == SL_Function_Code_Read_Profile)) {
        uint32_t tempCfgLen = 0;
        tempCfgLen = (gw485RxBuffer[3] << 8) | gw485RxBuffer[4];

        if (tempCfgLen < maxReadCfgLen && tempCfgLen > 0) {
            state = readCfgLengthGW;
            frameLength = 8 + tempCfgLen;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < functionCodeGWMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ readRegStartAddressGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define ReadRegStartAddressGWMaxLen 7
uint8_t analysisReadRegStartAddressGW(void)
{
    if (gw485RxBufferIndex >= ReadRegStartAddressGWMaxLen) {
        /* ȡĴʼַ */
        uint16_t resAddrLen = ((gw485RxBuffer[5] << 8) | gw485RxBuffer[6]);
        if (resAddrLen < maxReadResAddrLen) {
            state = readRegStartNumberGW;
            return TRUE;
        }
        log_error("resAddrLen : %d \n", resAddrLen);
    }

    if (gw485RxBufferIndex < ReadRegStartAddressGWMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ readRegStartNumberGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define ReadRegStartNumberMaxLen 9
uint8_t analysisReadRegStartNumberGW(void)
{
    /* crcУ */
    if (gw485RxBufferIndex >= ReadRegStartNumberMaxLen) {
        uint16_t tempCrc = 0;
        tempCrc = (gw485RxBuffer[frameLength - 3] << 8) | gw485RxBuffer[frameLength - 2];

        if (tempCrc == checkModebusCrc(gw485RxBuffer, frameLength - 3)) {
            state = crcCheckBitGW;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < ReadRegStartNumberMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**     
  * @brief  ״̬ crcCheckBitGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisCrcCheckBitGW(void)
{
    /* ־УУ */
    if (gw485RxBufferIndex == frameLength) {
        if (gw485RxBuffer[frameLength - 1] == 0x16) {
            state = endFlagGW;
            return TRUE;
        }
    }

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ endFlagGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisEndFlagGW(device_handle device)
{
    /* ΪȡĴ */
    if (gw485RxBuffer[2] == SL_Function_Code_Read_Register) {
        GW_MsgProcFunc_Read_Register(device, gw485RxBuffer, frameLength);
    }

    /* ΪдĴ */
    if (gw485RxBuffer[2] == SL_Function_Code_Write_Register) {
        GW_MsgProcFunc_Write_Register(device, gw485RxBuffer, frameLength);
    }

    /* Ϊ·ļ */
    else if (gw485RxBuffer[9] == SL_Function_Code_Distribution_Profile) {
        GW_MsgProcFunc_Distribution_Profile(device, gw485RxBuffer, frameLength);
    }

    /* Ϊȡļ */
    else if (gw485RxBuffer[9] == SL_Function_Code_Read_Profile) {
        GW_MsgProcFunc_Read_Profile(device, gw485RxBuffer, frameLength);
    }

    //buff
    state = wait;
    gw485RxBufferIndex = 0;
    return 1;
}

/**
  * @brief  ״̬ writeRegStartAddressGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define WriteRegStartAddressGWMaxLen 7
uint8_t analysisWriteRegStartAddressGW(void)
{
    if (gw485RxBufferIndex >= 7) {
        /* дĴʼַ */
        uint16_t resAddrLen = ((gw485RxBuffer[5] << 8) | gw485RxBuffer[6]);
        if (resAddrLen < maxReadResAddrLen) {
            frameLength = 10 + 2 * resAddrLen;
            state = writeRegStartNumberGW;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < WriteRegStartAddressGWMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ writeRegStartNumberGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define WriteRegStartNumberGWMaxLen (frameLength - 1)
uint8_t analysisWriteRegStartNumberGW(void)
{
    /* crcУ */
    if (gw485RxBufferIndex >= WriteRegStartNumberGWMaxLen) {
        uint16_t tempCrc = 0;
        tempCrc = ((uint16_t)gw485RxBuffer[frameLength - 3] << 8) | (uint16_t)gw485RxBuffer[frameLength - 2];

        if (tempCrc == checkModebusCrc(gw485RxBuffer, frameLength - 3)) {
            state = crcCheckBitGW;
            return TRUE;
        }
        // log_error("tempCrc : %x \n", tempCrc);
        // log_error("checkModebusCrc : %x \n", checkModebusCrc(gw485RxBuffer, frameLength - 3));
    }

    if (gw485RxBufferIndex < WriteRegStartNumberGWMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ cfgFramesNumGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define CfgFramesNumGWMaxLen 7
uint8_t analysisCfgFramesNumGW(void)
{
    /* ·ļݳ */
    if (gw485RxBufferIndex >= CfgFramesNumGWMaxLen) {
        uint32_t tempLen = 0;
        tempLen = (gw485RxBuffer[12] << 8) | gw485RxBuffer[13];

        if (tempLen > 0 && tempLen <= maxDistributionCfgLen) {
            state = cfgLengthGW;
            frameLength = 10 + tempLen;
            // log_error("cfgFramesNumSL error : tempLen = %d \n", tempLen);
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < CfgFramesNumGWMaxLen) {
        return FALSE;
    }
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ cfgLengthGW
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define CfgLengthGWMaxLen (frameLength - 1)
uint8_t analysisCfgLengthGW(void)
{
    /* crcУ */
    if (gw485RxBufferIndex >= CfgLengthGWMaxLen) {
        uint16_t tempCrc = 0;
        tempCrc = (gw485RxBuffer[frameLength - 3] << 8) | gw485RxBuffer[frameLength - 2];

        if (tempCrc == checkModebusCrc(gw485RxBuffer, frameLength - 3)) {
            state = crcCheckBitGW;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < CfgLengthGWMaxLen) {
        return FALSE;
    }

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ 
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define ReadCfgLengthGWMaxLen (frameLength - 1)
uint8_t analysisReadCfgLengthGW(void)
{
    /* crcУ */
    if (gw485RxBufferIndex >= ReadCfgLengthGWMaxLen) {
        uint16_t tempCrc = 0;
        tempCrc = (gw485RxBuffer[frameLength - 3] << 8) | gw485RxBuffer[frameLength - 2];

        if (tempCrc == checkModebusCrc(gw485RxBuffer, frameLength - 3)) {
            state = crcCheckBitGW;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < ReadCfgLengthGWMaxLen) {
        return FALSE;
    }

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ startFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisStartFlagSL(void)
{
    /* У˿ں */
    if (gw485RxBuffer[2] < 10 && gw485RxBuffer[2] >= 0) {
        // log_info("portSL\n");
        state = portSL;
        return TRUE;
    }

    // log_error("portSL %d \n", gw485RxBuffer[2]);
    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ portSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define PortSLMaxLen 5
uint8_t analysisPortSL(void)
{
    /* ݰĶ */
    if (gw485RxBufferIndex >= PortSLMaxLen) {
        uint32_t tempLen = 0;
        tempLen = (gw485RxBuffer[3] << 8) | gw485RxBuffer[4];

        if (tempLen > 0 && tempLen <= 1024) {
            // log_info("lengthSL\n");
            state = lengthSL;
            frameLength = 6 + tempLen;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < PortSLMaxLen) {
        return FALSE;
    }
    
    // log_error("lengthSL %d \n", (gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ lengthSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define LengthSLMaxLen frameLength
uint8_t analysisLengthSL(void)
{
    /* ֡β */
    if (gw485RxBufferIndex >= LengthSLMaxLen) {
        if (gw485RxBuffer[LengthSLMaxLen - 1] == 0x17) {
            // log_info("endFlagSL\n");
            state = endFlagSL;
            return TRUE;
        }
    }

    if (gw485RxBufferIndex < LengthSLMaxLen) {
        return FALSE;
    }
    // log_info("endFlagSL %d\n", gw485RxBuffer[LengthSLMaxLen - 1]);

    state = wait;
    gw485RxBufferIndex--;
    memcpy(gw485RxBuffer, gw485RxBuffer + 1, gw485RxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ endFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisEndFlagSL(void)
{
    uint32_t tempLen = 0;
    tempLen = (gw485RxBuffer[3] << 8) | gw485RxBuffer[4];

    //ϵͳڴ治㣬ȥǰ
    if (xPortGetFreeHeapSize() < tempLen + 1024) {
        goto err;
    }

    /* ڲͬͨ˿ڣĽṹ岻ͬ */
    uint8_t *Buff;
    if (gw485RxBuffer[2] > 0 && gw485RxBuffer[2] < 5) {
        Buff = (uint8_t *)pvPortMalloc(tempLen + queueUartSendInfoSize);
        queueUartSendInfo *sendBuff = (queueUartSendInfo *)Buff;
        sendBuff->length = tempLen;
        sendBuff->data = Buff + queueUartSendInfoSize;
        memcpy((char *)sendBuff->data, (char *)&gw485RxBuffer[5], tempLen);
    }
    else {
        Buff = (uint8_t *)pvPortMalloc(tempLen + queueTimeShareSendInfoSize);
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->length = tempLen;
        sendBuff->data = Buff + queueTimeShareSendInfoSize;
        memcpy((char *)sendBuff->data, (char *)&gw485RxBuffer[5], tempLen);
        // printf("\n\n\n\n\n%s\n", (char *)gw485RxBuffer);
        // printf("%s\n%d\n\n\n\n\n\n", sendBuff->data, tempLen);
    }

    /* ͨͬĶ˿ڽݷ͵ͬĵط */
    if (gw485RxBuffer[2] == 1) {
        if (uxQueueSpacesAvailable(J1_485_Queue)) {
            xQueueSend(J1_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 2) {
        if (uxQueueSpacesAvailable(J2_485_Queue)) {
            xQueueSend(J2_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 3) {
        if (uxQueueSpacesAvailable(J3_485_Queue)) {
            xQueueSend(J3_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 4) {
        if (uxQueueSpacesAvailable(J4_485_Queue)) {
            xQueueSend(J4_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 5) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ5;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 6) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ6;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 7) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ7;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 8) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ8;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 9) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ9;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (gw485RxBuffer[2] == 0) {
        queueTimeShareSendInfo *sendBuff = (queueTimeShareSendInfo *)Buff;
        sendBuff->connectPort = connectJ0;
        if (uxQueueSpacesAvailable(J5_0_485_Queue)) {
            xQueueSend(J5_0_485_Queue, &Buff, 10);    
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }

err:
    //buff
    state = wait;
    gw485RxBufferIndex = 0;
    return 1;
}

/**
  * @brief  ȡĴ
  * @param
  * @retval
  */
void GW_MsgProcFunc_Read_Register(device_handle device,uint8_t *buff, uint16_t buffLen)
{
    uint16_t Start_Address_16 = (gw485RxBuffer[3] << 8) | gw485RxBuffer[4];
    uint16_t Register_Number_16 = (gw485RxBuffer[5] << 8) | gw485RxBuffer[6];

    /* ȡ */
    uint16_t reply_Data_Content[maxReadResAddrLen] = {0};
    for ( uint16_t pos = 0;  pos < Register_Number_16; pos++) {
        for (uint16_t var = 0; var < sizeof(g_RegTblR) / sizeof(SL_RegProcTable); var++) {
            if (g_RegTblR[var].regId == (Start_Address_16 + pos)) {
                reply_Data_Content[pos] = g_RegTblR[var].pRegProc(NULL);
            }
        }
    }

    /* ʼڴ */
    //ϵͳڴ治㣬ȥǰ
    uint32_t tempLen = 8 + 2 * Register_Number_16 + queueUartSendInfoSize;
    if (xPortGetFreeHeapSize() < tempLen + 1024) {
        return;
    }

    uint8_t *Buff = (uint8_t *)pvPortMalloc(tempLen);
    queueUartSendInfo *sendBuff = (queueUartSendInfo *)Buff;
    sendBuff->length = 8 + 2 * Register_Number_16;
    sendBuff->data = Buff + sizeof(queueUartSendInfo);

    uint8_t *replay_pack = sendBuff->data;
    /* ʼ־ */
    *(replay_pack)      =  'G';
    *(replay_pack + 1)  =  'W';
    
    /*  */
    replay_pack += 2;
    *replay_pack = SL_Function_Code_Read_Register;

    /* ظֽ */
    replay_pack += 1;
    *replay_pack = (uint8_t)((Register_Number_16 * 2) >> 8);
    *(replay_pack + 1) = (uint8_t)(Register_Number_16 * 2);

    /* ظ */
    replay_pack += 2;
    for (uint8_t var = 0; var < Register_Number_16 * 2; var++) {
        if (0 == (var & 0x01)) {
            *(replay_pack + var) = (uint8_t)(reply_Data_Content[var / 2] >> 8);
        } else {
            *(replay_pack + var) = (uint8_t)(reply_Data_Content[var / 2]);
        }
    }

    /* Уλ */
    replay_pack += Register_Number_16 * 2;
    uint16_t packLen = 2 + 1 + 2 + Register_Number_16 * 2 + 3;
    uint16_t crc_temp = checkModebusCrc(sendBuff->data, packLen - 3);
    *replay_pack = (uint8_t)(crc_temp >> 8);
    replay_pack += 1;
    *replay_pack = (uint8_t)crc_temp;

    /* ־ */
    replay_pack += 1;
    *replay_pack = 0x16;

    /* пռ䣬򽫷 */
    if (uxQueueSpacesAvailable(upward_uart_Queue)) {
        xQueueSend(upward_uart_Queue, &Buff, 10);    
    }
    else {
        vPortFree(Buff);
    }
}

/**
  * @brief  дĴ
  * @param
  * @retval
  */
void GW_MsgProcFunc_Write_Register(device_handle device, void *pMsg, uint32_t MsgLen)
{
    uint16_t Register_Start_Address = (gw485RxBuffer[3] << 8) | gw485RxBuffer[4];
    uint16_t Register_Number = (gw485RxBuffer[5] << 8) | gw485RxBuffer[6];

    /* ָеݶȡ */
    uint16_t content[maxWriteResAddrLen] = {0};
    for (uint16_t var = 0; var < Register_Number; var++) {
        content[var] = gw485RxBuffer[7 + 2 * var] << 8 | gw485RxBuffer[7 + 2 * var + 1];
    }

    /* д뵽Ĵ */
    for ( uint16_t pos = 0;  pos < Register_Number; pos++) {
        for (uint16_t i = 0; i < sizeof(g_RegTblW) / sizeof(SL_RegProcTable); i++) {
            // debug_printf("g_RegTblW[i].regId : %x \n", g_RegTblW[i].regId);
            // debug_printf("Register_Start_Address : %x \n", (Register_Start_Address + pos));
            if (g_RegTblW[i].regId == (Register_Start_Address + pos)) {
                g_RegTblW[i].pRegProc(&content[pos]);
            }
        }
    }
}

/**
  * @brief  ·ļ
  * @param
  * @retval
  */
void GW_MsgProcFunc_Distribution_Profile(device_handle device, void *pMsg, uint32_t MsgLen)
{
    log_info("in GW_MsgProcFunc_Distribution_Profile\n");
}

/**
  * @brief  ȡļ
  * @param
  * @retval
  */
void GW_MsgProcFunc_Read_Profile(device_handle device, void *pMsg, uint32_t MsgLen)
{
    log_info("in GW_MsgProcFunc_Read_Profile\n");
}

/**
  * @brief  ״̬
  * @param 
  * @retval 
  */
void stateMachine(device_handle device)
{
    if (state == wait) {
        if (analysisWait() == TRUE) {
            gw485RxTime = xTaskGetTickCount();
        }
    }

    else if (state == startFlagGW) {
        analysisStartFlagGW();
    }

    else if (state == functionCodeGW) {
        analysisFunctionCodeGW();
    }

    else if (state == readRegStartAddressGW) {
        analysisReadRegStartAddressGW();
    }

    else if (state == readRegStartNumberGW) {
        analysisReadRegStartNumberGW();
    }

    else if (state == crcCheckBitGW) {
        analysisCrcCheckBitGW();
    }

    else if (state == endFlagGW) {
        analysisEndFlagGW(device);
    }

    else if (state == writeRegStartAddressGW) {
        analysisWriteRegStartAddressGW();
    }

    else if (state == writeRegStartNumberGW) {
        analysisWriteRegStartNumberGW();
    }

    else if (state == cfgFramesNumGW) {
        analysisCfgFramesNumGW();
    }

    else if (state == cfgLengthGW) {
        analysisCfgLengthGW();
    }

    else if (state == readCfgLengthGW) {
        analysisReadCfgLengthGW();
    }

    else if (state == startFlagSL) {
        analysisStartFlagSL();
    }

    else if (state == portSL) {
        analysisPortSL();
    }

    else if (state == lengthSL) {
        analysisLengthSL();
    }

    else if (state == endFlagSL) {
        analysisEndFlagSL();
    }
}


/**
  * @brief  modbuscrcУ
  * @param  *arr_buff   ҪУ
  *         len         ݳ
  * @retval crc         УĽ
  */
uint16_t checkModebusCrc(uint8_t *arr_buff, uint8_t len)
{
    uint16_t crc = 0xFFFF;
    uint16_t i, j;
    for (j = 0; j < len; ++j) {
        crc = crc ^ (*arr_buff++);
        for (i = 0; i < 8; ++i) {
            if ((crc&0x0001) > 0) {
                crc = crc >> 1;
                crc = crc ^ 0xa001;
            }
            else {
                crc = crc >> 1;
            }
        }
    }
    return crc;
}

/**
  * @brief  õλȡĵĲֵ
  * @param  lastTick  ϴλȡĽֵ
  * @retval
  */
uint32_t getTickDiff(uint32_t lastTick)
{
    int64_t temp;
    temp = xTaskGetTickCount() - lastTick;

    /* ֵֵ¼ */
    if (temp < 0) {
        temp = portMAX_DELAY - lastTick + xTaskGetTickCount();
    }
    
    return temp;
}

/**
  * @brief  ģ鷢
  * @param  device ڽģ鷢ݵĴ豸
  * @retval
  */
void upwardUartDataAnalysis(device_handle device)
{
    /* ÿκִ10ms */
    static uint32_t tickstart = 0U;
    tickstart = xTaskGetTickCount();

    /* 2Sδһ֡ݣ */
    if (getTickDiff(gw485RxTime) >= tick_2S) {
        gw485RxTime = xTaskGetTickCount();
        gw485RxBufferIndex = 0;
        state = wait;
    }

    while (uart_dev_char_present(device) == 1 && ((xTaskGetTickCount() - tickstart) < 5)) {
        gw485RxBuffer[gw485RxBufferIndex++] = uart_dev_in_char(device);
        stateMachine(device);
    }

    if (uart_dev_char_present(device) != 1 && state != wait) {        
        stateMachine(device);
    }
}
